/**
 * \file mcql/core/mbrpeptidemeasurements.cpp
 * \date 27/10/2024
 * \author Olivier Langella
 * \brief peptide extracted measures in MBR mode model in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "mbrpeptidemeasurements.h"

#include "msrungroup.h"
#include <pappsomspp/processing/detection/tracepeaklist.h>
#include <pappsomspp/pappsoexception.h>

mcql::MbrPeptideMeasurements::MbrPeptideMeasurements(
  const mcql::PeptideSp &peptide_sp)
  : msp_peptide(peptide_sp)
{
  if(msp_peptide.get() == nullptr)
    {
      throw pappso::PappsoException("msp_peptide.get() == nullptr");
    }
}

mcql::MbrPeptideMeasurements::~MbrPeptideMeasurements()
{
}


void
mcql::MbrPeptideMeasurements::prepareMeasurements(
  const pappso::MsRunId &targeted_msrun,
  const mcql::MsRunGroup *msrun_group_p,
  const pappso::MsRunRetentionTime<QString> &msrun_retention_time,
  const QuantificationMethodSp &quantification_method,
  double ni_min_abundance)
{
  // for one peptide
  //  1) charge states
  //  2) isotope list
  std::vector<std::uint8_t> charge_states =
    msp_peptide.get()->getAllObservedChargeStateList();

  double consensus_rt = msp_peptide.get()->getConsensusRetentionTime();

  if(consensus_rt > 0)
    {
      if(msrun_retention_time.isAligned())
        {
          consensus_rt =
            msrun_retention_time.translateAligned2OriginalRetentionTime(
              consensus_rt);
        }
    }
  const pappso::PeptideNaturalIsotopeList *p_peptide_natural_isotope_list =
    nullptr;
  if(ni_min_abundance > 0)
    {
      p_peptide_natural_isotope_list =
        msp_peptide.get()->getPeptideNaturalIsotopeList();
    }
  for(std::uint8_t charge : charge_states)
    {
      double mz = msp_peptide.get()->getPappsoPeptideSp().get()->getMz(charge);

      pappso::XicCoordSPtr best_xic_coord;
      if(msrun_group_p->getIonMobilityGridSp().get() != nullptr)
        {
          best_xic_coord =
            msp_peptide.get()
              ->getBestIonMobilityXicCoordToExtractOverallMsRunGroup(
                *msrun_group_p, targeted_msrun, charge);
        }
      else
        {
          best_xic_coord =
            msp_peptide.get()->getBestXicCoordSPtrForChargeInMsRunGroup(
              msrun_group_p, charge);
        }

      if(best_xic_coord.get() == nullptr)
        {
          // We can not quantify it in this msrun group
        }
      else
        {
          pappso::XicCoordSPtr xic_coord =
            best_xic_coord.get()->initializeAndClone();
          xic_coord.get()->rtTarget = consensus_rt;
          xic_coord.get()->mzRange =
            quantification_method.get()->getXicExtractionMzRange(mz);


          if(p_peptide_natural_isotope_list != nullptr)
            { // isotopes
              std::vector<pappso::PeptideNaturalIsotopeAverageSp>
                natural_isotope_average_list =
                  msp_peptide.get()->getPeptideNaturalIsotopeAverageSpList(
                    quantification_method.get()
                      ->getXicExtractionMeanPrecisionPtr(),
                    charge,
                    ni_min_abundance);


              for(pappso::PeptideNaturalIsotopeAverageSp isotope_average_sp :
                  natural_isotope_average_list)
                {

                  double mz = isotope_average_sp.get()->getMz();
                  pappso::XicCoordSPtr xicCoord =
                    best_xic_coord.get()->initializeAndClone();
                  xicCoord.get()->rtTarget = consensus_rt;
                  xicCoord.get()->mzRange =
                    quantification_method.get()->getXicExtractionMzRange(mz);


                  m_measurementList.push_back({charge,
                                               xicCoord,
                                               PeakQualityCategory::nomatch,
                                               nullptr,
                                               isotope_average_sp});
                }
            }
          else
            { // no isotope
              m_measurementList.push_back({charge,
                                           xic_coord,
                                           PeakQualityCategory::nomatch,
                                           nullptr,
                                           nullptr});
            }
        }
    }
}

void
mcql::MbrPeptideMeasurements::pushBackXicCoordList(
  std::vector<pappso::XicCoordSPtr> &xic_coord_list) const
{
  for(auto &measure : m_measurementList)
    {
      if(measure.msp_xicCoord.get()->rtTarget > 0)
        {
          xic_coord_list.push_back(measure.msp_xicCoord);
        }
    }
}

void
mcql::MbrPeptideMeasurements::detectQuantifyPeaks(
  const mcql::QuantificationMethodSp &quantification_method)
{

  qDebug();
  pappso::TracePeakList peak_list;
  std::vector<std::uint8_t> all_charge_states =
    msp_peptide.get()->getAllObservedChargeStateList();

  for(auto &measure_one : m_measurementList)
    {
      qDebug();
      double rt_target = measure_one.msp_xicCoord.get()->rtTarget;
      peak_list.clear();
      quantification_method.get()
        ->getTraceDetectionInterfaceCstSPtr()
        .get()
        ->detect(
          *(measure_one.msp_xicCoord.get()->xicSptr.get()), peak_list, false);

      qDebug();
      measure_one.m_peakQualityCategory = PeakQualityCategory::missed;
      if(rt_target > 0)
        {
          qDebug();
          auto it_best_matched_peak = pappso::findTracePeakGivenRt(
            peak_list.begin(), peak_list.end(), rt_target);

          if(it_best_matched_peak != peak_list.end())
            {
              measure_one.m_tracePeakSp =
                it_best_matched_peak->makeTracePeakCstSPtr();
              measure_one.m_peakQualityCategory = PeakQualityCategory::b;
            }
        }
    }
}

const mcql::PeptideSp &
mcql::MbrPeptideMeasurements::getPeptideSp() const
{
  return msp_peptide;
}

const std::vector<mcql::PeptideMeasurements::Measurement> &
mcql::MbrPeptideMeasurements::getMeasurementList() const
{
  return m_measurementList;
}
