/**
 * \file mcql/core/msrungroup.cpp
 * \date 10/10/2024
 * \author Olivier Langella
 * \brief group msrun together in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "msrungroup.h"
#include <pappsomspp/exception/exceptionnotfound.h>

mcql::MsRunGroup::MsRunGroup(
  const QString &id, const std::vector<mcql::MsRunPeptideListSp> &msrun_list)
  : m_id(id), m_msRunPeptideListSpList(msrun_list)
{
  if(m_msRunPeptideListSpList.size() > 0)
    {
      m_hasTimsTofMobilityIndex = m_msRunPeptideListSpList.front()
                                    .get()
                                    ->getMsRunSp()
                                    .get()
                                    ->hasTimsTofMobilityIndex();
    }
}

mcql::MsRunGroup::MsRunGroup(const mcql::MsRunGroup &other)
  : m_id(other.m_id), m_msRunPeptideListSpList(other.m_msRunPeptideListSpList)
{
  m_hasTimsTofMobilityIndex = other.m_hasTimsTofMobilityIndex;
}

mcql::MsRunGroup::~MsRunGroup()
{
}

void
mcql::MsRunGroup::quantify(
  const QString &quantify_id,
  const QString &tmp_dir,
  double retention_time_range_seconds,
  pappso::UiMonitorInterface &m_uiMonitor,
  const QuantificationMethodSp &quantification_method,
  double ni_minimum_abundance,
  const std::vector<mcql::PeptideSp> &peptide_sp_list,
  const std::vector<mcql::QuantificationResultWriterInterfaceSp> &writer_list)
  const
{

  std::shared_ptr<pappso::MsRunRetentionTime<QString>>
    retention_time_reference_sp = nullptr;
  if(msp_msRunPeptideListAlignmentReference.get() != nullptr)
    {

      msp_msRunPeptideListAlignmentReference.get()->quantify(
        this,
        tmp_dir,
        retention_time_range_seconds,
        m_uiMonitor,
        quantification_method,
        ni_minimum_abundance);


      for(auto &writer_sp : writer_list)
        {

          msp_msRunPeptideListAlignmentReference.get()
            ->writeQuantificationResults(
              m_uiMonitor, writer_sp, quantify_id, m_id);
        }

      retention_time_reference_sp =
        msp_msRunPeptideListAlignmentReference->buildMsRunRetentionTimeSp(
          msp_alignmentMethod);
    }

  for(const mcql::MsRunPeptideListSp &msrun_peptide_list :
      m_msRunPeptideListSpList)
    {
      if(msrun_peptide_list.get() ==
         msp_msRunPeptideListAlignmentReference.get())
        continue;
      msrun_peptide_list.get()->quantify(this,
                                         tmp_dir,
                                         retention_time_range_seconds,
                                         m_uiMonitor,
                                         quantification_method,
                                         ni_minimum_abundance);

      if(retention_time_reference_sp != nullptr)
        {
          // launch retention time alignment
          auto retention_time_sp =
            msrun_peptide_list.get()->buildMsRunRetentionTimeSp(
              msp_alignmentMethod);

          retention_time_sp.get()->align(*(retention_time_reference_sp.get()));
        }

      for(auto &writer_sp : writer_list)
        {

          msrun_peptide_list.get()->writeQuantificationResults(
            m_uiMonitor, writer_sp, quantify_id, m_id);
        }
    }

  if(retention_time_reference_sp != nullptr)
    {

      m_uiMonitor.setStatus("Collecting peak apex");
      for(const mcql::MsRunPeptideListSp &msrun_peptide_list :
          m_msRunPeptideListSpList)
        {
          msrun_peptide_list.get()->collectPeptidePeakRetentionTime(
            retention_time_reference_sp.get());
        }

      m_uiMonitor.setStatus(
        QObject::tr("Computing consensus retention time (%1)")
          .arg(peptide_sp_list.size()));
      for(const mcql::PeptideSp &peptide_sp : peptide_sp_list)
        {
          peptide_sp.get()->computeConsensusRetentionTime();
        }

      for(const mcql::MsRunPeptideListSp &msrun_peptide_list :
          m_msRunPeptideListSpList)
        {
          const mcql::MsRun *msrun_p =
            msrun_peptide_list.get()->getMsRunSp().get();
          std::vector<mcql::PeptideSp> peptide_mbr_list;
          for(const mcql::PeptideSp &peptide_sp : peptide_sp_list)
            {
              if(peptide_sp.get()->isNotObservedInMsRunSp(msrun_p))
                {
                  peptide_mbr_list.push_back(peptide_sp);
                }
            }
          msrun_peptide_list.get()->quantifyMatchBetweenRun(
            this,
            peptide_mbr_list,
            tmp_dir,
            retention_time_range_seconds,
            m_uiMonitor,
            quantification_method,
            ni_minimum_abundance);


          for(auto &writer_sp : writer_list)
            {

              msrun_peptide_list.get()->writeMbrQuantificationResults(
                m_uiMonitor, writer_sp, quantify_id, m_id);
            }
        }
    }
}

void
mcql::MsRunGroup::setAlignmentMethodSp(
  const mcql::AlignmentMethodSp &alignment_method_sp,
  const QString &msrun_ref_id)
{
  auto it = find_if(m_msRunPeptideListSpList.begin(),
                    m_msRunPeptideListSpList.end(),
                    [msrun_ref_id](const mcql::MsRunPeptideListSp &msrun) {
                      if(msrun.get()
                           ->getMsRunSp()
                           .get()
                           ->getMsRunReaderSPtr()
                           .get()
                           ->getMsRunId()
                           .get()
                           ->getXmlId() == msrun_ref_id)
                        return true;
                      return false;
                    });
  if(it != m_msRunPeptideListSpList.end())
    {
      msp_alignmentMethod                    = alignment_method_sp;
      msp_msRunPeptideListAlignmentReference = *it;
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("msrun_id %1 not found in group %2")
          .arg(msrun_ref_id)
          .arg(m_id));
    }
}

bool
mcql::MsRunGroup::contains(const mcql::MsRun *msrun_p) const
{
  auto it = find_if(m_msRunPeptideListSpList.begin(),
                    m_msRunPeptideListSpList.end(),
                    [msrun_p](const mcql::MsRunPeptideListSp &msrun) {
                      return (msrun.get()->getMsRunSp().get() == msrun_p);
                    });
  return (it != m_msRunPeptideListSpList.end());
}

void
mcql::MsRunGroup::setIonMobilityGridSp(
  const std::shared_ptr<pappso::IonMobilityGrid> &ion_mobility_grid_sp)
{
  msp_ionMobilityGrid = ion_mobility_grid_sp;
}

bool
mcql::MsRunGroup::hasTimsTofMobilityIndex() const
{
  return m_hasTimsTofMobilityIndex;
}

const std::shared_ptr<pappso::IonMobilityGrid> &
mcql::MsRunGroup::getIonMobilityGridSp() const
{
  return msp_ionMobilityGrid;
}
