/**
 * \file mcql/core/msrunpeptidelist.cpp
 * \date 25/09/2024
 * \author Olivier Langella
 * \brief msrun model including fragmented peptides in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "msrunpeptidelist.h"
#include <pappsomspp/xicextractor/msrunxicextractorfactory.h>
#include <pappsomspp/pappsoexception.h>
#include <QtConcurrent>

mcql::MsRunPeptideList::MsRunPeptideList(mcql::MsRunSp msrun)
{
  msp_msRun = msrun;
}

mcql::MsRunPeptideList::MsRunPeptideList(const mcql::MsRunPeptideList &other)
{
  msp_msRun = other.msp_msRun;
}

mcql::MsRunPeptideList::~MsRunPeptideList()
{
}

void
mcql::MsRunPeptideList::addPeptideSpectrumIndexObservation(
  mcql::PeptideSp peptide_sp, std::size_t spectrum_index, std::uint8_t charge)
{

  auto it = m_peptideObservationList.insert(
    {peptide_sp.get(), std::make_shared<mcql::PeptideObservation>(peptide_sp)});
  mcql::PrecursorSp precursor_sp =
    msp_msRun.get()->getPrecursorSPtrBySpectrumIndex(spectrum_index);
  it.first->second.get()->addObservation(
    {spectrum_index, charge, precursor_sp, true});

  peptide_sp.get()->addMsRunXicCoordCharge(
    {msp_msRun.get(),
     precursor_sp.get()->getXicCoordSPtr(),
     charge,
     precursor_sp.get()->getIntensity()});
}

void
mcql::MsRunPeptideList::addPeptideScanNumberObservation(
  mcql::PeptideSp peptide_sp, std::size_t scan_number, std::uint8_t charge)
{

  auto it = m_peptideObservationList.insert(
    {peptide_sp.get(), std::make_shared<mcql::PeptideObservation>(peptide_sp)});
  mcql::PrecursorSp precursor_sp =
    msp_msRun.get()->getPrecursorSPtrByScanNumber(scan_number);
  it.first->second.get()->addObservation(
    {scan_number, charge, precursor_sp, false});

  peptide_sp.get()->addMsRunXicCoordCharge(
    {msp_msRun.get(),
     precursor_sp.get()->getXicCoordSPtr(),
     charge,
     precursor_sp.get()->getIntensity()});
}

void
mcql::MsRunPeptideList::quantify(
  const mcql::MsRunGroup *msrun_group_p,
  const QString &tmp_dir,
  double retention_time_range_seconds,
  pappso::UiMonitorInterface &ui_monitor,
  const QuantificationMethodSp &quantification_method,
  double ni_minimum_abundance)
{
  m_peptideMeasurementsList.clear();
  ui_monitor.setStatus(QString("Starting quantification for msrun %1")
                         .arg(msp_msRun.get()
                                ->getMsRunReaderSPtr()
                                ->getMsRunId()
                                .get()
                                ->getFileName()));

  qDebug();
  pappso::MsRunXicExtractorFactory::getInstance().setTmpDir(tmp_dir);

  qDebug();
  pappso::MsRunXicExtractorFactory::getInstance()
    .setMsRunXicExtractorFactoryType(
      pappso::MsRunXicExtractorFactoryType::diskbuffer);

  qDebug();
  ui_monitor.setStatus(QString("Preparing extraction list"));
  std::vector<pappso::XicCoordSPtr> xic_coord_list = buildXicCoordList(
    msrun_group_p, quantification_method, ni_minimum_abundance);

  ui_monitor.setStatus(QString("Preparing extraction list done"));

  ui_monitor.setStatus(QString("Building XIC extractor"));
  pappso::MsRunXicExtractorInterfaceSp extractor_pwiz =
    pappso::MsRunXicExtractorFactory::getInstance().buildMsRunXicExtractorSp(
      msp_msRun.get()->getMsRunReaderSPtr());

  qDebug();
  extractor_pwiz->setXicExtractMethod(
    quantification_method.get()->getXicExtractMethod());

  qDebug();

  extractor_pwiz.get()->setRetentionTimeAroundTarget(
    retention_time_range_seconds);

  qDebug();
  pappso::FilterInterfaceCstSPtr csp_filter_suite =
    std::make_shared<const pappso::FilterSuite>(
      quantification_method.get()->getXicFilterSuite());

  qDebug();
  extractor_pwiz.get()->setPostExtractionTraceFilterCstSPtr(csp_filter_suite);

  qDebug();

  ui_monitor.setStatus(QString("Building XIC extractor done"));
  ui_monitor.setStatus(QString("Extracting"));
  extractor_pwiz.get()->extractXicCoordSPtrListParallelized(ui_monitor,
                                                            xic_coord_list);

  qDebug();

  detectQuantifyPeaks(ui_monitor, quantification_method);


  ui_monitor.setStatus(QString("Quantification for msrun %1 done")
                         .arg(msp_msRun.get()
                                ->getMsRunReaderSPtr()
                                ->getMsRunId()
                                .get()
                                ->getFileName()));
}


std::vector<pappso::XicCoordSPtr>
mcql::MsRunPeptideList::buildXicCoordList(
  const mcql::MsRunGroup *msrun_group_p,
  const QuantificationMethodSp &quantification_method,
  double ni_minimum_abundance)
{
  std::vector<pappso::XicCoordSPtr> xic_coord_list;

  for(auto pair_peptide : m_peptideObservationList)
    {
      mcql::PeptideObservationSp peptide_events_sp = pair_peptide.second;

      addPeptideObservation2XicCoordList(msrun_group_p,
                                         quantification_method,
                                         pair_peptide.second,
                                         xic_coord_list,
                                         ni_minimum_abundance);
      // peptide_events_sp.get()
    }


  return xic_coord_list;
}

void
mcql::MsRunPeptideList::addPeptideObservation2XicCoordList(
  const mcql::MsRunGroup *msrun_group_p,
  const QuantificationMethodSp &quantification_method,
  const mcql::PeptideObservationSp &peptide_events_sp,
  std::vector<pappso::XicCoordSPtr> &xic_coord_list,
  double ni_minimum_abundance)
{
  PeptideMeasurementsSp peptide_measurements =
    std::make_shared<mcql::PeptideMeasurements>(peptide_events_sp);

  m_peptideMeasurementsList.push_back(peptide_measurements);

  peptide_measurements.get()->prepareMeasurements(
    *(msp_msRun.get()->getMsRunReaderSPtr().get()->getMsRunId().get()),
    msrun_group_p,
    quantification_method,
    ni_minimum_abundance);
  peptide_measurements.get()->pushBackXicCoordList(xic_coord_list);
}

void
mcql::MsRunPeptideList::detectQuantifyPeaks(
  pappso::UiMonitorInterface &m_uiMonitor,
  const QuantificationMethodSp &quantification_method)
{

  m_uiMonitor.setStatus(QString("Detect and quantify peak"));
  /*
    for(auto &measure : m_peptideMeasurementsList)
      {
        measure->detectQuantifyPeaks(quantification_method);
      }
  */

  std::function<void(const mcql::PeptideMeasurementsSp &)>
    mapdetectQuantifyPeaks =
      [quantification_method](const mcql::PeptideMeasurementsSp &measure) {
        measure->detectQuantifyPeaks(quantification_method);
      };


  QFuture<void> res =
    QtConcurrent::map<std::vector<mcql::PeptideMeasurementsSp>::iterator>(
      m_peptideMeasurementsList.begin(),
      m_peptideMeasurementsList.end(),
      mapdetectQuantifyPeaks);
  res.waitForFinished();


  m_uiMonitor.setStatus(QString("Detect and quantify peak done"));
}

void
mcql::MsRunPeptideList::writeQuantificationResults(
  pappso::UiMonitorInterface &ui_monitor,
  mcql::QuantificationResultWriterInterfaceSp writer_sp,
  const QString &quantify_id,
  const QString &withingroup) const
{
  writer_sp.get()->setMsRunSp(msp_msRun);
  writer_sp.get()->setQuantificationId(quantify_id);
  writer_sp.get()->setGroupId(withingroup);

  ui_monitor.setStatus(
    QString("Writing quantification results for %1 %2 msrun %3")
      .arg(quantify_id)
      .arg(withingroup)
      .arg(msp_msRun.get()
             ->getMsRunReaderSPtr()
             .get()
             ->getMsRunId()
             .get()
             ->getSampleName()));

  ui_monitor.setTotalSteps(m_peptideMeasurementsList.size());
  writer_sp.get()->startPeptideMeasurements();
  for(const mcql::PeptideMeasurementsSp &measures : m_peptideMeasurementsList)
    {
      writer_sp.get()->write(*(measures.get()), msp_msRunRetentionTime.get());
      ui_monitor.count();
    }
  ui_monitor.setTotalSteps(0);
  writer_sp.get()->endPeptideMeasurements();
}

const mcql::MsRunSp &
mcql::MsRunPeptideList::getMsRunSp() const
{
  return msp_msRun;
}

void
mcql::MsRunPeptideList::flushChromatogramTraces()
{

  for(auto &measure : m_peptideMeasurementsList)
    {
      measure.get()->flushXics();
    }
}

std::shared_ptr<pappso::MsRunRetentionTime<QString>> &
mcql::MsRunPeptideList::buildMsRunRetentionTimeSp(
  const mcql::AlignmentMethodSp &alignment_method)
{
  msp_msRunRetentionTime =
    std::make_shared<pappso::MsRunRetentionTime<QString>>(
      msp_msRun->getRetentionTimeLine());

  msp_msRunRetentionTime.get()->setMs2MedianFilter(
    pappso::FilterMorphoMedian(alignment_method.get()->getMs2TendencyWindow()));
  msp_msRunRetentionTime.get()->setMs2MeanFilter(
    pappso::FilterMorphoMean(alignment_method.get()->getMs2SmoothingWindow()));
  msp_msRunRetentionTime.get()->setMs1MeanFilter(
    pappso::FilterMorphoMean(alignment_method.get()->getMs1SmoothingWindow()));


  for(const mcql::PeptideMeasurementsSp &measures : m_peptideMeasurementsList)
    {
      QString peptide_id = measures.get()
                             ->getPeptideObservationSp()
                             .get()
                             ->getPeptideSp()
                             .get()
                             ->getId();
      for(auto &one_xic_measure : measures.get()->getMeasurementList())
        {
          if((one_xic_measure.m_peakQualityCategory ==
              PeakQualityCategory::aa) ||
             (one_xic_measure.m_peakQualityCategory == PeakQualityCategory::a))
            {
              msp_msRunRetentionTime.get()->addPeptideAsSeamark(
                peptide_id,
                one_xic_measure.m_tracePeakSp.get()->getMaxXicElement().x,
                one_xic_measure.m_tracePeakSp.get()->getMaxXicElement().y);
            }
        }
    }

  msp_msRunRetentionTime.get()->computeSeamarks();

  return msp_msRunRetentionTime;
}

void
mcql::MsRunPeptideList::collectPeptidePeakRetentionTime(
  pappso::MsRunRetentionTime<QString> *msrun_retention_time_reference_p) const
{
  for(const mcql::PeptideMeasurementsSp &measures : m_peptideMeasurementsList)
    {
      try
        {
          mcql::Peptide *peptide_p = measures.get()
                                       ->getPeptideObservationSp()
                                       .get()
                                       ->getPeptideSp()
                                       .get();
          peptide_p->setReferenceMsRunRetentionTimePtr(
            msrun_retention_time_reference_p);
          for(auto &one_xic_measure : measures.get()->getMeasurementList())
            {
              peptide_p->addAlignedPeakMeasurement(
                one_xic_measure, *(msp_msRunRetentionTime.get()));
            }
        }
      catch(const pappso::PappsoException &error)
        {
          throw pappso::PappsoException(
            QObject::tr(
              "error collecting peak retention time in msrun %1 :\n%2")
              .arg(
                msp_msRun.get()->getMsRunReaderSPtr()->getMsRunId()->getXmlId())
              .arg(error.qwhat()));
        }
    }
}


void
mcql::MsRunPeptideList::quantifyMatchBetweenRun(
  const mcql::MsRunGroup *msrun_group_p,
  const std::vector<mcql::PeptideSp> &peptide_mbr_list,
  const QString &tmp_dir,
  double retention_time_range_seconds,
  pappso::UiMonitorInterface &ui_monitor,
  const QuantificationMethodSp &quantification_method,
  double ni_minimum_abundance)
{
  try
    {
      m_peptideMeasurementsList.clear();
      ui_monitor.setStatus(QString("Starting quantification for msrun %1 (MBR)")
                             .arg(msp_msRun.get()
                                    ->getMsRunReaderSPtr()
                                    ->getMsRunId()
                                    .get()
                                    ->getFileName()));

      qDebug();
      pappso::MsRunXicExtractorFactory::getInstance().setTmpDir(tmp_dir);

      qDebug();
      pappso::MsRunXicExtractorFactory::getInstance()
        .setMsRunXicExtractorFactoryType(
          pappso::MsRunXicExtractorFactoryType::diskbuffer);

      qDebug();
      ui_monitor.setStatus(QString("MBR Preparing extraction list (%1)")
                             .arg(peptide_mbr_list.size()));


      std::vector<pappso::XicCoordSPtr> xic_coord_list;

      for(auto &peptide_sp : peptide_mbr_list)
        {
          /*
          mcql::PeptideObservationSp peptide_events_sp = pair_peptide.second;

          addPeptideObservation2XicCoordList(quantification_method,
                                             pair_peptide.second,
                                             xic_coord_list,
                                             ni_minimum_abundance);
                                             */
          addMbrPeptideMeasurementsSp2XicCoordList(msrun_group_p,
                                                   quantification_method,
                                                   peptide_sp,
                                                   xic_coord_list,
                                                   ni_minimum_abundance);
          // peptide_events_sp.get()
        }


      ui_monitor.setStatus(QString("MBR Preparing extraction list done (%1 %2)")
                             .arg(m_mbrPeptideMeasurementsList.size())
                             .arg(xic_coord_list.size()));

      ui_monitor.setStatus(QString("MBR Building XIC extractor"));
      pappso::MsRunXicExtractorInterfaceSp extractor_pwiz =
        pappso::MsRunXicExtractorFactory::getInstance()
          .buildMsRunXicExtractorSp(msp_msRun.get()->getMsRunReaderSPtr());

      qDebug();
      extractor_pwiz->setXicExtractMethod(
        quantification_method.get()->getXicExtractMethod());

      qDebug();

      extractor_pwiz.get()->setRetentionTimeAroundTarget(
        retention_time_range_seconds);

      qDebug();
      pappso::FilterInterfaceCstSPtr csp_filter_suite =
        std::make_shared<const pappso::FilterSuite>(
          quantification_method.get()->getXicFilterSuite());

      qDebug();
      extractor_pwiz.get()->setPostExtractionTraceFilterCstSPtr(
        csp_filter_suite);

      qDebug();

      ui_monitor.setStatus(QString("MBR Building XIC extractor done"));
      ui_monitor.setStatus(QString("MBR Extracting"));
      extractor_pwiz.get()->extractXicCoordSPtrListParallelized(ui_monitor,
                                                                xic_coord_list);

      qDebug();

      mbrDetectQuantifyPeaks(ui_monitor, quantification_method);


      ui_monitor.setStatus(QString("MBR Quantification for msrun %1 done")
                             .arg(msp_msRun.get()
                                    ->getMsRunReaderSPtr()
                                    ->getMsRunId()
                                    .get()
                                    ->getFileName()));
    }
  catch(const pappso::PappsoException &error)
    {
      throw pappso::PappsoException(
        QObject::tr("error in %1 :\n%2").arg(__FUNCTION__).arg(error.qwhat()));
    }
}


void
mcql::MsRunPeptideList::addMbrPeptideMeasurementsSp2XicCoordList(
  const mcql::MsRunGroup *msrun_group_p,
  const mcql::QuantificationMethodSp &quantification_method,
  const mcql::PeptideSp &peptide_sp,
  std::vector<pappso::XicCoordSPtr> &xic_coord_list,
  double ni_minimum_abundance)
{
  MbrPeptideMeasurementsSp mbr_peptide_measurements =
    std::make_shared<mcql::MbrPeptideMeasurements>(peptide_sp);

  m_mbrPeptideMeasurementsList.push_back(mbr_peptide_measurements);

  mbr_peptide_measurements.get()->prepareMeasurements(
    *(msp_msRun.get()->getMsRunReaderSPtr().get()->getMsRunId().get()),
    msrun_group_p,
    *(msp_msRunRetentionTime.get()),
    quantification_method,
    ni_minimum_abundance);
  mbr_peptide_measurements.get()->pushBackXicCoordList(xic_coord_list);
}

void
mcql::MsRunPeptideList::mbrDetectQuantifyPeaks(
  pappso::UiMonitorInterface &m_uiMonitor,
  const mcql::QuantificationMethodSp &quantification_method)
{

  m_uiMonitor.setStatus(QString("MBR Detect and quantify peak"));
  /*
    for(auto &measure : m_peptideMeasurementsList)
      {
        measure->detectQuantifyPeaks(quantification_method);
      }
  */

  std::function<void(const mcql::MbrPeptideMeasurementsSp &)>
    mapdetectQuantifyPeaks =
      [quantification_method](const mcql::MbrPeptideMeasurementsSp &measure) {
        measure->detectQuantifyPeaks(quantification_method);
      };


  QFuture<void> res =
    QtConcurrent::map<std::vector<mcql::MbrPeptideMeasurementsSp>::iterator>(
      m_mbrPeptideMeasurementsList.begin(),
      m_mbrPeptideMeasurementsList.end(),
      mapdetectQuantifyPeaks);
  res.waitForFinished();


  m_uiMonitor.setStatus(QString("Detect and quantify peak done"));
}


void
mcql::MsRunPeptideList::writeMbrQuantificationResults(
  pappso::UiMonitorInterface &ui_monitor,
  mcql::QuantificationResultWriterInterfaceSp writer_sp,
  const QString &quantify_id,
  const QString &withingroup) const
{
  writer_sp.get()->setMsRunSp(msp_msRun);
  writer_sp.get()->setQuantificationId(quantify_id);
  writer_sp.get()->setGroupId(withingroup);

  ui_monitor.setStatus(
    QString("Writing MBR quantification results for %1 %2 msrun %3")
      .arg(quantify_id)
      .arg(withingroup)
      .arg(msp_msRun.get()
             ->getMsRunReaderSPtr()
             .get()
             ->getMsRunId()
             .get()
             ->getSampleName()));

  ui_monitor.setTotalSteps(m_peptideMeasurementsList.size());
  writer_sp.get()->startPeptideMeasurements();
  try
    {
      for(const mcql::MbrPeptideMeasurementsSp &measures :
          m_mbrPeptideMeasurementsList)
        {
          writer_sp.get()->write(*(measures.get()),
                                 msp_msRunRetentionTime.get());
          ui_monitor.count();
        }
    }
  catch(const pappso::PappsoException &error)
    {
      throw pappso::PappsoException(
        QObject::tr("error in %1 :\n%2").arg(__FUNCTION__).arg(error.qwhat()));
    }
  ui_monitor.setTotalSteps(0);
  writer_sp.get()->endPeptideMeasurements();
}
