/**
 * \file mcql/core/peptidemeasurements.cpp
 * \date 26/09/2024
 * \author Olivier Langella
 * \brief peptide extracted measures model in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "peptidemeasurements.h"
#include "peptideobservation.h"
#include "msrungroup.h"
#include <pappsomspp/processing/detection/tracepeaklist.h>
#include <pappsomspp/pappsoexception.h>

namespace mcql
{
PeptideMeasurements::PeptideMeasurements(
  const mcql::PeptideObservationSp &peptide)
{
  msp_peptideObservation = peptide;
}

PeptideMeasurements::PeptideMeasurements(const PeptideMeasurements &other)
{
  msp_peptideObservation = other.msp_peptideObservation;
}

PeptideMeasurements::~PeptideMeasurements()
{
}
} // namespace mcql

void
mcql::PeptideMeasurements::prepareMeasurements(
  const pappso::MsRunId &targeted_msrun,
  const mcql::MsRunGroup *msrun_group_p,
  const QuantificationMethodSp &quantification_method,
  double ni_min_abundance)
{
  // for one peptide
  //  1) charge states
  //  2) isotope list
  std::vector<std::uint8_t> charge_states = msp_peptideObservation.get()
                                              ->getPeptideSp()
                                              .get()
                                              ->getAllObservedChargeStateList();

  double best_rt = msp_peptideObservation.get()->getBestRtOverallChargeStates();


  pappso::XicCoordSPtr best_xic_coord =
    msp_peptideObservation.get()->getBestXicCoord();

  const pappso::PeptideNaturalIsotopeList *p_peptide_natural_isotope_list =
    nullptr;
  if(ni_min_abundance > 0)
    {
      p_peptide_natural_isotope_list = msp_peptideObservation.get()
                                         ->getPeptideSp()
                                         .get()
                                         ->getPeptideNaturalIsotopeList();
    }

  if(best_xic_coord.get() == nullptr)
    {
      throw pappso::PappsoException("best_xic_coord.get() == nullptr");
    }

  for(std::uint8_t charge : charge_states)
    {
      double mz = msp_peptideObservation.get()
                    ->getPeptideSp()
                    .get()
                    ->getPappsoPeptideSp()
                    .get()
                    ->getMz(charge);

      if(msrun_group_p->getIonMobilityGridSp().get() != nullptr)
        {

          // Ion mobility TimsTOF coordinates
          pappso::XicCoordSPtr best_xic_coord =
            msp_peptideObservation.get()->getBestXicCoordSPtrForCharge(charge);

          if(best_xic_coord.get() == nullptr)
            {
              // now in this ms run, we do not have ion mobility index for
              // this charge we must chek out in other msrun for this charge
              best_xic_coord =
                msp_peptideObservation.get()
                  ->getPeptideSp()
                  .get()
                  ->getBestIonMobilityXicCoordToExtractOverallMsRunGroup(
                    *msrun_group_p, targeted_msrun, charge);
            }
          if(best_xic_coord.get() == nullptr)
            continue;

          best_xic_coord.get()->rtTarget = best_rt;
        }


      if(p_peptide_natural_isotope_list == nullptr)
        {
          // no isotope

          pappso::XicCoordSPtr xic_coord = best_xic_coord->initializeAndClone();
          xic_coord.get()->rtTarget      = best_rt;
          xic_coord.get()->mzRange =
            quantification_method.get()->getXicExtractionMzRange(mz);


          m_measurementList.push_back({charge,
                                       xic_coord,
                                       PeakQualityCategory::nomatch,
                                       nullptr,
                                       nullptr});
        }
      else
        {

          std::vector<pappso::PeptideNaturalIsotopeAverageSp>
            natural_isotope_average_list =
              msp_peptideObservation.get()
                ->getPeptideSp()
                .get()
                ->getPeptideNaturalIsotopeAverageSpList(
                  quantification_method.get()
                    ->getXicExtractionMeanPrecisionPtr(),
                  charge,
                  ni_min_abundance);

          for(pappso::PeptideNaturalIsotopeAverageSp isotope_average_sp :
              natural_isotope_average_list)
            {
              pappso::XicCoordSPtr xic_coord =
                best_xic_coord->initializeAndClone();
              xic_coord.get()->rtTarget = best_rt;
              xic_coord.get()->mzRange =
                quantification_method.get()->getXicExtractionMzRange(
                  isotope_average_sp.get()->getMz());


              m_measurementList.push_back({charge,
                                           xic_coord,
                                           PeakQualityCategory::nomatch,
                                           nullptr,
                                           isotope_average_sp});
            }
        }
    }
}

void
mcql::PeptideMeasurements::pushBackXicCoordList(
  std::vector<pappso::XicCoordSPtr> &xic_coord_list) const
{
  for(auto &measure : m_measurementList)
    {
      xic_coord_list.push_back(measure.msp_xicCoord);
    }
}

void
mcql::PeptideMeasurements::detectQuantifyPeaks(
  const QuantificationMethodSp &quantification_method)
{

  qDebug();
  pappso::TracePeakList peak_list;
  std::vector<double> observed_rt_list =
    msp_peptideObservation.get()
      ->getObservedRetentionTimesOverallChargeStates();

  std::vector<std::uint8_t> observed_charge_states =
    msp_peptideObservation->getObservedChargeStates();

  for(auto &measure_one : m_measurementList)
    {
      qDebug();
      peak_list.clear();
      std::size_t nb_peaks = 0;
      quantification_method.get()
        ->getTraceDetectionInterfaceCstSPtr()
        .get()
        ->detect(
          *(measure_one.msp_xicCoord.get()->xicSptr.get()), peak_list, false);

      qDebug();
      auto it_best_matched_peak = findBestTracePeakGivenRtList(
        peak_list.begin(), peak_list.end(), observed_rt_list, nb_peaks);

      qDebug();
      measure_one.m_peakQualityCategory       = PeakQualityCategory::missed;
      bool direct_charge_observation_in_msrun = true;
      if(observed_charge_states.end() == find(observed_charge_states.begin(),
                                              observed_charge_states.end(),
                                              measure_one.m_charge))
        {
          direct_charge_observation_in_msrun = false;
        }
      if(it_best_matched_peak != peak_list.end())
        {
          measure_one.m_tracePeakSp =
            it_best_matched_peak->makeTracePeakCstSPtr();
          if(nb_peaks == 1)
            {

              measure_one.m_peakQualityCategory = PeakQualityCategory::za;
              if(direct_charge_observation_in_msrun)
                measure_one.m_peakQualityCategory = PeakQualityCategory::a;

              if(observed_rt_list.size() > 1)
                {
                  measure_one.m_peakQualityCategory = PeakQualityCategory::zaa;
                  if(direct_charge_observation_in_msrun)
                    measure_one.m_peakQualityCategory = PeakQualityCategory::aa;
                }
            }
          else if(nb_peaks > 1)
            {
              measure_one.m_peakQualityCategory = PeakQualityCategory::zab;
              if(direct_charge_observation_in_msrun)
                measure_one.m_peakQualityCategory = PeakQualityCategory::ab;
            }
        }
    }
}

const std::vector<mcql::PeptideMeasurements::Measurement> &
mcql::PeptideMeasurements::getMeasurementList() const
{
  return m_measurementList;
}

const mcql::PeptideObservationSp &
mcql::PeptideMeasurements::getPeptideObservationSp() const
{
  return msp_peptideObservation;
}

void
mcql::PeptideMeasurements::flushXics()
{

  for(auto &measure_one : m_measurementList)
    {
      measure_one.msp_xicCoord.get()->xicSptr = nullptr;
    }
}
