
/**
 * \file mcql/input/masschroqmlparser.h
 * \date 23/09/2024
 * \author Olivier Langella
 * \brief masschroqML parser for MassChroqLight
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#pragma once


#include <pappsomspp/processing/xml/xmlstreamreaderinterface.h>
#include <pappsomspp/processing/uimonitor/uimonitorinterface.h>
#include <QFileInfo>
#include <map>
#include <pappsomspp/peptide/peptide.h>
#include "../core/msrungroup.h"
#include "../core/msrunpeptidelist.h"
#include "../core/protein.h"
#include "../core/alignmentmethod.h"
#include "../core/quantificationmethod.h"
#include "../output/quantificationresultwriterinterface.h"

namespace mcql
{
/**
 * @todo write docs
 */
class MassChroqMlParser : public pappso::XmlStreamReaderInterface
{
  public:
  /**
   * Default constructor
   */
  MassChroqMlParser(pappso::UiMonitorInterface &ui_monitor,
                    const QString &tmp_dir_name);

  /**
   * Destructor
   */
  virtual ~MassChroqMlParser();


  virtual bool readFile(const QString &fileName) override;

  protected:
  virtual void readStream() override;

  private:
  void readRawdata();
  void readProjectParameters();
  void readGroups();
  void readProteinList();
  void readPeptideList();
  void readAlignments();
  void readAlignmentMethods();
  void readQuantificationMethods();
  void readQuantification();
  void read_peptide();
  std::pair<QString, QString> read_data_file();
  void read_psimod(pappso::Peptide &peptide);
  void read_observed_in(const mcql::PeptideSp &peptide_sp,
                        QStringList &msrun_id_list);
  void read_quantification_method();
  void
  read_xic_filters(const mcql::QuantificationMethodSp &quantification_method);
  void read_peak_detection(
    const mcql::QuantificationMethodSp &quantification_method);
  void
  read_xic_filter(const mcql::QuantificationMethodSp &quantification_method);
  void read_detection_zivy(
    const mcql::QuantificationMethodSp &quantification_method);
  void read_detection_moulon(
    const mcql::QuantificationMethodSp &quantification_method);
  void read_quantify();
  void
  readInQuantify(const QString &quantify_id,
                 const QString &withingroup,
                 const mcql::QuantificationMethodSp &quantification_method);
  void read_quantification_results();
  mcql::QuantificationResultWriterInterfaceSp read_quantification_result();
  void read_protein();
  void read_group();
  void read_alignment_method();
  void read_align();
  void read_ms2(const mcql::AlignmentMethodSp &align_method_sp);


  bool ensureFilePathExists(const QString &output_file);
  QString ensureFileExtension(const QString &output_file,
                              const QString &file_extension);


  private:
  const QString &m_tmpDirName;
  const double m_retentionTimeRangeSeconds = 300;
  pappso::UiMonitorInterface &m_uiMonitor;

  std::map<QString, mcql::MsRunGroupSp> m_msRunGroupSpList;
  std::map<QString, mcql::MsRunPeptideListSp> m_msfileList;
  std::map<QString, mcql::AlignmentMethodSp> m_alignmentMethodSpList;
  std::map<QString, mcql::QuantificationMethodSp> m_quantificationMethodSpList;


  std::map<QString, mcql::ProteinSp> m_proteinMap;
  std::vector<mcql::PeptideSp> m_peptideStore;
  std::vector<mcql::QuantificationResultWriterInterfaceSp>
    m_quantificationWriterList;

  std::shared_ptr<pappso::IonMobilityGrid> msp_ionMobilityGrid;
  
   QFile *mpa_cborfile = nullptr;
};

} // namespace mcql
