/**
 * \file mcql/masschroqlightcli.cpp
 * \date 23/09/2024
 * \author Olivier Langella
 * \brief simple MassChroQ XIC extraction
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/


#include "masschroqlightcli.h"
#include <pappsomspp/pappsoexception.h>
#include <QDateTime>
#include <QCommandLineParser>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <pappsomspp/processing/uimonitor/uimonitortextpercent.h>
#include "../config.h"
#include "input/masschroqmlparser.h"
#include <QThreadPool>


MassChroqLightCli::MassChroqLightCli(QObject *parent) : QObject(parent)
{
  // get the instance of the main application
  m_mcqLightApp = QCoreApplication::instance();
  // setup everything here
  // create any global objects
  // setup debug and warning mode
}

void
MassChroqLightCli::aboutToQuitApp()
{
}
void
MassChroqLightCli::quit()
{
  // you can do some cleanup here
  // then do emit finished to signal CoreApplication to quit
  emit finished();
}
void
MassChroqLightCli::run()
{

  try
    {
      const QDateTime dt_begin = QDateTime::currentDateTime();
      const QString masschroq_dir_path(QCoreApplication::applicationDirPath());

      QCommandLineParser parser;
      parser.setApplicationDescription(
        "Perform Mass Chromatogram Quantification as indicated in the XML "
        "input "
        "FILE. For additional information, see the MassChroQ Homepage : "
        "http://pappso.inra.fr/bioinfo/masschroq/");
      parser.addHelpOption();
      parser.addVersionOption();
      parser.addPositionalArgument("MassChroqML",
                                   tr("MassChroQ XML input file to execute."));

      //    out << "  -c, --cpus integer\t" << endl;
      QCommandLineOption cpusOption(
        QStringList() << "c"
                      << "cpus",
        tr("sets the number of cpus you want to use"),
        tr("number"));
      parser.addOption(cpusOption);

      QCommandLineOption tmpOption(
        QStringList() << "t"
                      << "tmpdir",
        tr("use DIRECTORY as temporary working direcory for masschroq : "
           "temporary "
           "files generated by masschroq during execution time (one file or "
           "several slices at a time per xml data file being analysed) will go "
           "there. By default DIRECTORY is the current working directory of "
           "masschroq. Use this option if you want to set another one."),
        tr("directory"));
      parser.addOption(tmpOption);


      QCommandLineOption mcqmlFile(QStringList() << "m"
                                                 << "mcqml",
                                   tr("MassChroqML input file"),
                                   tr("filename"));
      parser.addOption(mcqmlFile);


      parser.process(*m_mcqLightApp);


      pappso::UiMonitorTextPercent ui_monitor(ConsoleOut::mcq_cout());

      ui_monitor.setTitle(
        QObject::tr("MassChroQlite version %1").arg(MASSCHROQ_VERSION));


      QString tmpDirName = parser.value(tmpOption);
      if(tmpDirName.isEmpty())
        {
          /// by default, temporary files go in the current working directory
          tmpDirName = QDir::currentPath();
        }
      else
        {
        }

      m_tmpDirName = tmpDirName;

      QString cpusStr = parser.value(cpusOption);
      if(cpusStr.isEmpty())
        {
          // mcqout() << "masschroqml_validation cpusStr.isEmpty()" << endl;
          setCpuNumber(1);
          // mcqout() << "masschroqml_validation cpusStr.isEmpty() 2"<< endl;
        }
      else
        {
          setCpuNumber(cpusStr.toUInt());
        }

      QString filename = parser.value(mcqmlFile);

      if(!filename.isEmpty())
        {
          readMassChroqMlFile(ui_monitor, filename);
        }
    }
  catch(pappso::PappsoException &error)
    {
      ConsoleOut::mcq_cerr() << "MassChroQ encountered an error:" << Qt::endl;
      ConsoleOut::mcq_cerr() << error.qwhat() << Qt::endl;

      exit(1);
      m_mcqLightApp->exit(1);
    }
  qDebug() << "before final quit()";

  // you must call quit when complete or the program will stay in the
  // messaging loop
  quit();
}


void
MassChroqLightCli::setCpuNumber(uint cpu_number)
{

  qDebug() << " Quantificator::setCpuNumber" << cpu_number;
  uint ideal_number_of_thread = (uint)QThread::idealThreadCount();
  if(cpu_number > ideal_number_of_thread)
    {
      cpu_number = ideal_number_of_thread;
    }
  else
    {
      QThreadPool::globalInstance()->setMaxThreadCount(cpu_number);
    }
  m_cpuNumber = cpu_number;
  ConsoleOut::mcq_cout() << QObject::tr("%1 CPUs used").arg(cpu_number)
                         << Qt::endl;
}


void
MassChroqLightCli::readMassChroqMlFile(pappso::UiMonitorInterface &ui_monitor,
                                       const QString &filename)
{

  QFile file(QFileInfo(filename).absoluteFilePath());
  if(!file.open(QIODevice::ReadOnly))
    {
      throw pappso::PappsoException(
        QObject::tr("Unable to open xml masschroqML file '%1' :\n%2\n")
          .arg(QFileInfo(filename).absoluteFilePath())
          .arg(file.errorString()));
    }
  mcql::MassChroqMlParser parser(ui_monitor, m_tmpDirName);


  ui_monitor.setStatus(QObject::tr("Parsing XML input file '%1'")
                         .arg(QFileInfo(filename).absoluteFilePath()));

  if(parser.readFile(QFileInfo(filename).absoluteFilePath()))
    {
      file.close();
    }
  else
    {
      file.close();
      throw pappso::PappsoException(
        QObject::tr("error reading masschroqML input file '%1' :\n")
          .arg(QFileInfo(filename).absoluteFilePath())
          .append(parser.errorString()));
    }
  ConsoleOut::mcq_cout() << "MassChroQ : DONE on file '"
                         << QFileInfo(filename).absoluteFilePath() << "'"
                         << Qt::endl;
}
