
/**
 * \file mcql/output/quantificationresultcboroutput.cpp
 * \date 24/11/2024
 * \author Olivier Langella
 * \brief quantification result cbor writer for MassChroqLight
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/


#include "quantificationresultcboroutput.h"
#include "../core/peptideobservation.h"

mcql::QuantificationResultCborOutput::QuantificationResultCborOutput(
  QIODevice *device)
{
  mpa_writer = new QCborStreamWriter(device);
  mpa_writer->startMap();
}

mcql::QuantificationResultCborOutput::~QuantificationResultCborOutput()
{
  mpa_writer->endMap();

  delete mpa_writer;
}

void
mcql::QuantificationResultCborOutput::writePeptideProteinList(
  const std::vector<mcql::PeptideSp> &peptide_list)
{
  mpa_writer->append(QLatin1String("peptide_list"));
  mpa_writer->startArray(peptide_list.size());
  for(const mcql::PeptideSp &peptide_sp : peptide_list)
    {

      mpa_writer->startMap();
      mpa_writer->append(QLatin1String("id"));
      mpa_writer->append(peptide_sp.get()->getId());
      mpa_writer->append(QLatin1String("proforma"));
      mpa_writer->append(
        peptide_sp.get()->getPappsoPeptideSp().get()->toProForma());
      mpa_writer->append(QLatin1String("proteins"));
      const std::vector<mcql::ProteinSp> &protein_list =
        peptide_sp.get()->getProteinSpList();
      mpa_writer->startArray(protein_list.size());
      for(const mcql::ProteinSp &protein_sp : protein_list)
        {
          mpa_writer->append(protein_sp.get()->getId());
        }

      mpa_writer->endArray();


      mpa_writer->endMap();
    }

  mpa_writer->endArray();
}

void
mcql::QuantificationResultCborOutput::endPeptideMeasurements()
{
  mpa_writer->endArray();
  mpa_writer->endMap();
}

void
mcql::QuantificationResultCborOutput::startPeptideMeasurements()
{
  mpa_writer->append(QLatin1String("quantification"));
  mpa_writer->startMap();
  mpa_writer->append(QLatin1String("id"));
  mpa_writer->append(m_currentQuantifyId);
  mpa_writer->append(QLatin1String("group_id"));
  mpa_writer->append(m_currentGroupId);
  mpa_writer->append(QLatin1String("msrun"));
  writeMsRun(*(msp_currentMsRun.get()));
  mpa_writer->startArray();
}

void
mcql::QuantificationResultCborOutput::writeMsRun(const mcql::MsRun &msrun)
{
  mpa_writer->startMap(3);
  mpa_writer->append(QLatin1String("id"));
  mpa_writer->append(
    msrun.getMsRunReaderSPtr().get()->getMsRunId().get()->getXmlId());
  mpa_writer->append(QLatin1String("filename"));
  mpa_writer->append(
    msrun.getMsRunReaderSPtr().get()->getMsRunId().get()->getFileName());
  mpa_writer->append(QLatin1String("sample"));
  mpa_writer->append(
    msrun.getMsRunReaderSPtr().get()->getMsRunId().get()->getSampleName());
  mpa_writer->endMap();
}


void
mcql::QuantificationResultCborOutput::write(
  const mcql::MbrPeptideMeasurements &mbr_peptide_measurements,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{
  mcql::PeptideSp peptide_sp = mbr_peptide_measurements.getPeptideSp();

  mpa_writer->startMap();
  mpa_writer->append(QLatin1String("peptide_id"));
  mpa_writer->append(peptide_sp.get()->getId());
  mpa_writer->append(QLatin1String("proforma"));
  mpa_writer->append(
    peptide_sp.get()->getPappsoPeptideSp().get()->toProForma());

  mpa_writer->append(QLatin1String("xics"));
  mpa_writer->startArray();
  for(const mcql::PeptideMeasurements::Measurement &measure :
      mbr_peptide_measurements.getMeasurementList())
    {
      // write(peptide_sp, measure, msrun_retention_time_p);
      writeMeasurement(measure, msrun_retention_time_p);
    }
  mpa_writer->endArray();

  mpa_writer->endMap();
}

void
mcql::QuantificationResultCborOutput::write(
  const mcql::PeptideMeasurements &peptide_measurements,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{
  mcql::PeptideSp peptide_sp =
    peptide_measurements.getPeptideObservationSp().get()->getPeptideSp();

  mpa_writer->startMap();
  mpa_writer->append(QLatin1String("peptide_id"));
  mpa_writer->append(peptide_sp.get()->getId());
  mpa_writer->append(QLatin1String("proforma"));
  mpa_writer->append(
    peptide_sp.get()->getPappsoPeptideSp().get()->toProForma());

  mpa_writer->append(QLatin1String("xics"));
  mpa_writer->startArray();
  for(const mcql::PeptideMeasurements::Measurement &measure :
      peptide_measurements.getMeasurementList())
    {
      // write(peptide_sp, measure, msrun_retention_time_p);
      writeMeasurement(measure, msrun_retention_time_p);
    }
  mpa_writer->endArray();

  mpa_writer->endMap();
}

void
mcql::QuantificationResultCborOutput::writeMeasurement(
  const mcql::PeptideMeasurements::Measurement &measurement,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{
  mpa_writer->startMap();
  mpa_writer->append(QLatin1String("charge"));
  mpa_writer->append(measurement.m_charge);

  mpa_writer->append(QLatin1String("mz"));
  mpa_writer->append(measurement.msp_xicCoord.get()->mzRange.getMz());

  if(measurement.m_peptideNaturalIsotopeAverageSp.get() != nullptr)
    {
      mpa_writer->append(QLatin1String("isotope"));
      mpa_writer->append(
        measurement.m_peptideNaturalIsotopeAverageSp.get()->getIsotopeNumber());

      mpa_writer->append(QLatin1String("rank"));
      mpa_writer->append(
        measurement.m_peptideNaturalIsotopeAverageSp.get()->getIsotopeRank());
    }

  if(m_isTraceOutput &&
     (measurement.msp_xicCoord.get()->xicSptr.get() != nullptr))
    {
      mpa_writer->append(QLatin1String("trace"));
      writeTrace(*(measurement.msp_xicCoord.get()->xicSptr.get()));
    }


  mpa_writer->append(QLatin1String("quality"));
  mpa_writer->append(QuantificationResultWriterInterface::enumToString(
    measurement.m_peakQualityCategory));
  if(measurement.m_tracePeakSp.get() != nullptr)
    {
      mpa_writer->append(QLatin1String("peak"));
      writeTracePeak(*(measurement.m_tracePeakSp.get()),
                     msrun_retention_time_p);
    }
  mpa_writer->endMap();
}

void
mcql::QuantificationResultCborOutput::writeTrace(const pappso::Trace &trace)
{
  mpa_writer->startMap(2);
  mpa_writer->append(QLatin1String("x"));
  mpa_writer->startArray(trace.size());
  for(double x : trace.xValues())
    {
      mpa_writer->append(x);
    }
  mpa_writer->endArray();


  mpa_writer->append(QLatin1String("y"));
  mpa_writer->startArray(trace.size());
  for(double y : trace.yValues())
    {
      mpa_writer->append(y);
    }
  mpa_writer->endArray();
  mpa_writer->endMap();
}

void
mcql::QuantificationResultCborOutput::writeTracePeak(
  const pappso::TracePeak &peak,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{
  mpa_writer->startMap();
  mpa_writer->append(QLatin1String("area"));
  mpa_writer->append(peak.getArea());

  mpa_writer->append(QLatin1String("max_intensity"));
  mpa_writer->append(peak.getMaxXicElement().y);

  mpa_writer->append(QLatin1String("rt"));
  mpa_writer->startArray();
  mpa_writer->append(peak.getLeftBoundary().x);
  mpa_writer->append(peak.getMaxXicElement().x);
  mpa_writer->append(peak.getRightBoundary().x);
  mpa_writer->endArray();

  if((msrun_retention_time_p != nullptr) && msrun_retention_time_p->isAligned())
    {
      mpa_writer->append(QLatin1String("aligned_rt"));
      mpa_writer->startArray();
      mpa_writer->append(
        msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
          peak.getLeftBoundary().x));
      mpa_writer->append(
        msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
          peak.getMaxXicElement().x));
      mpa_writer->append(
        msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
          peak.getRightBoundary().x));
      mpa_writer->endArray();
    }

  mpa_writer->endMap();
}
