
/**
 * \file mcql/output/quantificationresultodsoutput.h
 * \date 27/09/2024
 * \author Olivier Langella
 * \brief quantification result ods writer for MassChroqLight
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "quantificationresultodsoutput.h"
#include <pappsomspp/pappsoexception.h>
#include "../core/peptideobservation.h"


mcql::QuantificationResultOdsOutputBase::QuantificationResultOdsOutputBase()
{
}


mcql::QuantificationResultOdsOutputBase::~QuantificationResultOdsOutputBase()
{
  mpa_calcWriterInterface->close();

  delete mpa_calcWriterInterface;
}


mcql::QuantificationResultOdsOutput::QuantificationResultOdsOutput(
  const QString &ods_file)
{

  try
    {
      OdsDocWriter *ods_p     = new OdsDocWriter(ods_file);
      mpa_calcWriterInterface = ods_p;
    }
  catch(OdsException &error)
    {
      throw pappso::PappsoException(
        QObject::tr("Error writing ODS file %1 :\n%2")
          .arg(ods_file)
          .arg(error.qwhat()));
    }
}

mcql::QuantificationResultOdsOutput::~QuantificationResultOdsOutput()
{
}


mcql::QuantificationResultTsvOutput::QuantificationResultTsvOutput(
  const QString &output_directory)
{
  try
    {
      TsvDirectoryWriter *tsv_dir_p =
        new TsvDirectoryWriter(QDir(output_directory));
      tsv_dir_p->setFlushLines(true);
      mpa_calcWriterInterface = tsv_dir_p;
    }
  catch(OdsException &error)
    {
      throw pappso::PappsoException(
        QObject::tr("Error writing TSV ouput in directory %1 :\n%2")
          .arg(output_directory)
          .arg(error.qwhat()));
    }
}
mcql::QuantificationResultTsvOutput::~QuantificationResultTsvOutput()
{
}


void
mcql::QuantificationResultOdsOutputBase::startPeptideMeasurements()
{
  QString sheet_name =
    QString("peptides_%1_%2").arg(m_currentQuantifyId).arg(m_currentGroupId);
  try
    {
      if(m_currentSheetName == sheet_name)
        {
        }
      else
        {
          m_currentSheetName = sheet_name;
          mpa_calcWriterInterface->writeSheet(sheet_name);

          OdsTableSettings settings;
          settings.setVerticalSplit(1);
          mpa_calcWriterInterface->setCurrentOdsTableSettings(settings);

          mpa_calcWriterInterface->setCellAnnotation("quantification XML id");
          mpa_calcWriterInterface->writeCell("quantification");
          mpa_calcWriterInterface->setCellAnnotation(
            "group XML id (fraction name)");
          mpa_calcWriterInterface->writeCell("group");
          mpa_calcWriterInterface->setCellAnnotation(
            "MS run XML id (sample id)");
          mpa_calcWriterInterface->writeCell("msrun");
          mpa_calcWriterInterface->setCellAnnotation("MS run file path");
          mpa_calcWriterInterface->writeCell("msrunfile");
          mpa_calcWriterInterface->setCellAnnotation("XIC m/z");
          mpa_calcWriterInterface->writeCell("mz");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak maximum intensity retention time aligned");
          mpa_calcWriterInterface->writeCell("rt");
          mpa_calcWriterInterface->setCellAnnotation("peak maximum intensity");
          mpa_calcWriterInterface->writeCell("maxintensity");
          mpa_calcWriterInterface->setCellAnnotation("peak area");
          mpa_calcWriterInterface->writeCell("area");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak quality : how the peak was measured");
          mpa_calcWriterInterface->writeCell("peak quality");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak start retention time aligned");
          mpa_calcWriterInterface->writeCell("rtbegin");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak stop retention time aligned");
          mpa_calcWriterInterface->writeCell("rtend");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak start retention time not aligned");
          mpa_calcWriterInterface->writeCell("realrtbegin");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak apex (maximum intensity) retention time not aligned");
          mpa_calcWriterInterface->writeCell("realrtapex");
          mpa_calcWriterInterface->setCellAnnotation(
            "peak stop retention time not aligned");
          mpa_calcWriterInterface->writeCell("realrtend");
          mpa_calcWriterInterface->setCellAnnotation("peptide id");
          mpa_calcWriterInterface->writeCell("peptide");
          mpa_calcWriterInterface->setCellAnnotation("isotope label");
          mpa_calcWriterInterface->writeCell("label");
          mpa_calcWriterInterface->setCellAnnotation("peptide sequence");
          mpa_calcWriterInterface->writeCell("sequence");
          mpa_calcWriterInterface->setCellAnnotation("peptide charge");
          mpa_calcWriterInterface->writeCell("z");
          mpa_calcWriterInterface->setCellAnnotation(
            "peptide modifications (free text)");
          mpa_calcWriterInterface->writeCell("mods");


          mpa_calcWriterInterface->setCellAnnotation("natural isotope number");
          mpa_calcWriterInterface->writeCell("ninumber");
          mpa_calcWriterInterface->setCellAnnotation("natural isotope rank");
          mpa_calcWriterInterface->writeCell("nirank");
          mpa_calcWriterInterface->setCellAnnotation(
            "natural isotope theoretical ratio");
          mpa_calcWriterInterface->writeCell("niratio");


          mpa_calcWriterInterface->writeLine();
        }
    }
  catch(OdsException &ods_error)
    {
      throw pappso::PappsoException(
        QObject::tr("error writing quantification for group %1 :\n%2")
          .arg(m_currentGroupId)
          .arg(ods_error.qwhat()));
    }
}

void
mcql::QuantificationResultOdsOutputBase::write(
  const mcql::PeptideMeasurements &peptide_measurements,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{

  mcql::PeptideSp peptide_sp =
    peptide_measurements.getPeptideObservationSp().get()->getPeptideSp();
  for(const mcql::PeptideMeasurements::Measurement &measure :
      peptide_measurements.getMeasurementList())
    {
      write(peptide_sp, measure, msrun_retention_time_p);
    }
}

void
mcql::QuantificationResultOdsOutputBase::write(
  const mcql::MbrPeptideMeasurements &peptide_measurements,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{

  mcql::PeptideSp peptide_sp = peptide_measurements.getPeptideSp();
  for(const mcql::PeptideMeasurements::Measurement &measure :
      peptide_measurements.getMeasurementList())
    {
      write(peptide_sp, measure, msrun_retention_time_p);
    }
}

void
mcql::QuantificationResultOdsOutputBase::endPeptideMeasurements()
{
}

void
mcql::QuantificationResultOdsOutputBase::writePeptideProteinList(
  const std::vector<mcql::PeptideSp> &peptide_list)
{
  try
    {
      mpa_calcWriterInterface->writeSheet(
        QString("%1_proteins").arg(m_currentGroupId));
      //_p_writer->writeLine();
      mpa_calcWriterInterface->writeCell("peptide");
      mpa_calcWriterInterface->writeCell("ProForma");
      mpa_calcWriterInterface->writeCell("protein");
      mpa_calcWriterInterface->writeCell("protein_description");
      mpa_calcWriterInterface->writeLine();

      OdsTableSettings settings;
      settings.setVerticalSplit(1);
      mpa_calcWriterInterface->setCurrentOdsTableSettings(settings);


      for(const mcql::PeptideSp &peptide_sp : peptide_list)
        {
          for(const mcql::ProteinSp &protein_sp :
              peptide_sp.get()->getProteinSpList())
            {
              mpa_calcWriterInterface->writeCell(peptide_sp.get()->getId());
              mpa_calcWriterInterface->writeCell(
                peptide_sp.get()->getPappsoPeptideSp().get()->toProForma());
              mpa_calcWriterInterface->writeCell(protein_sp.get()->getId());
              mpa_calcWriterInterface->writeCell(
                protein_sp.get()->getDescription());
              mpa_calcWriterInterface->writeLine();
            }
        }
    }
  catch(OdsException &ods_error)
    {
      throw pappso::PappsoException(
        QObject::tr("error writing protein list for group %1 :\n%2")
          .arg(m_currentGroupId)
          .arg(ods_error.qwhat()));
    }
}


void
mcql::QuantificationResultOdsOutputBase::write(
  const mcql::PeptideSp &peptide_sp,
  const mcql::PeptideMeasurements::Measurement &measure,
  const pappso::MsRunRetentionTime<QString> *msrun_retention_time_p)
{
  mpa_calcWriterInterface->writeCell(m_currentQuantifyId);
  mpa_calcWriterInterface->writeCell(m_currentGroupId);
  mpa_calcWriterInterface->writeCell(msp_currentMsRun->getMsRunReaderSPtr()
                                       .get()
                                       ->getMsRunId()
                                       .get()
                                       ->getXmlId());
  mpa_calcWriterInterface->writeCell(msp_currentMsRun->getMsRunReaderSPtr()
                                       .get()
                                       ->getMsRunId()
                                       .get()
                                       ->getSampleName());
  mpa_calcWriterInterface->writeCell(
    measure.msp_xicCoord.get()->mzRange.getMz());

  if(measure.m_tracePeakSp.get() != nullptr)
    {
      if((msrun_retention_time_p == nullptr) ||
         (!msrun_retention_time_p->isAligned()))
        {
          mpa_calcWriterInterface->writeCell(
            measure.m_tracePeakSp.get()->getMaxXicElement().x);
        }
      else
        {

          mpa_calcWriterInterface->writeCell(
            msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
              measure.m_tracePeakSp.get()->getMaxXicElement().x));
        }
      mpa_calcWriterInterface->writeCell(
        measure.m_tracePeakSp.get()->getMaxXicElement().y);
      mpa_calcWriterInterface->writeCell(
        measure.m_tracePeakSp.get()->getArea());
    }
  else
    {

      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(0);
    }


  mpa_calcWriterInterface->writeCell(
    enumToString(measure.m_peakQualityCategory));


  if(measure.m_tracePeakSp.get() != nullptr)
    {

      if((msrun_retention_time_p == nullptr) ||
         (!msrun_retention_time_p->isAligned()))
        {
          mpa_calcWriterInterface->writeCell(
            measure.m_tracePeakSp.get()->getLeftBoundary().x);
          mpa_calcWriterInterface->writeCell(
            measure.m_tracePeakSp.get()->getRightBoundary().x);
        }
      else
        {
          mpa_calcWriterInterface->writeCell(
            msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
              measure.m_tracePeakSp.get()->getLeftBoundary().x));
          mpa_calcWriterInterface->writeCell(
            msrun_retention_time_p->translateOriginal2AlignedRetentionTime(
              measure.m_tracePeakSp.get()->getRightBoundary().x));
        }

      mpa_calcWriterInterface->writeCell(
        measure.m_tracePeakSp.get()->getLeftBoundary().x);
      mpa_calcWriterInterface->writeCell(
        measure.m_tracePeakSp.get()->getMaxXicElement().x);
      mpa_calcWriterInterface->writeCell(
        measure.m_tracePeakSp.get()->getRightBoundary().x);
    }
  else
    {

      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(0);

      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(0);
    }

  mpa_calcWriterInterface->writeCell(peptide_sp.get()->getId());
  mpa_calcWriterInterface->writeEmptyCell();
  mpa_calcWriterInterface->writeCell(
    peptide_sp.get()->getPappsoPeptideSp().get()->getSequence());
  mpa_calcWriterInterface->writeCell(measure.m_charge);


  mpa_calcWriterInterface->writeCell(peptide_sp.get()->getMods());

  if(measure.m_peptideNaturalIsotopeAverageSp.get() != nullptr)
    {
      mpa_calcWriterInterface->writeCell(
        (std::size_t)measure.m_peptideNaturalIsotopeAverageSp.get()
          ->getIsotopeNumber());
      mpa_calcWriterInterface->writeCell(
        (std::size_t)measure.m_peptideNaturalIsotopeAverageSp.get()
          ->getIsotopeRank());
      mpa_calcWriterInterface->writeCell(
        measure.m_peptideNaturalIsotopeAverageSp.get()->getIntensityRatio());
    }
  else
    {
      mpa_calcWriterInterface->writeCell(0);
      mpa_calcWriterInterface->writeCell(1);
      mpa_calcWriterInterface->writeCell(0);
    }

  mpa_calcWriterInterface->writeLine();
}
