/**
 * \file mcql/core/peptide.cpp
 * \date 24/09/2024
 * \author Olivier Langella
 * \brief peptide model in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "peptide.h"
#include <pappsomspp/pappsoexception.h>
#include "msrungroup.h"

namespace mcql
{
Peptide::Peptide(const QString &id,
                 pappso::PeptideSp peptide_sp,
                 const std::vector<ProteinSp> &protein_list)
  : m_id(id), m_proteinSpList(protein_list)
{
  msp_peptide = peptide_sp;
}

Peptide::Peptide(const Peptide &other)
  : m_id(other.m_id), m_proteinSpList(other.m_proteinSpList)
{
  msp_peptide = other.msp_peptide;
  m_mods      = other.m_mods;
}

Peptide::~Peptide()
{
  if(mpa_peptideNaturalIsotopeList != nullptr)
    delete mpa_peptideNaturalIsotopeList;
}

} // namespace mcql

const pappso::PeptideSp &
mcql::Peptide::getPappsoPeptideSp() const
{
  return msp_peptide;
}

void
mcql::Peptide::computeIsotopologues(double ni_min_abundance)
{

  if(ni_min_abundance > 0)
    {

      qDebug() << "if (_minimum_isotope_abundance > 0)";
      mpa_peptideNaturalIsotopeList =
        new pappso::PeptideNaturalIsotopeList(msp_peptide, 0.001);
    }
}

const pappso::PeptideNaturalIsotopeList *
mcql::Peptide::getPeptideNaturalIsotopeList() const
{
  return mpa_peptideNaturalIsotopeList;
}

const QString &
mcql::Peptide::getId() const
{
  return m_id;
}

void
mcql::Peptide::setMods(const QString &mods)
{
  m_mods = mods;
}

const QString &
mcql::Peptide::getMods() const
{
  return m_mods;
}

const std::vector<pappso::PeptideNaturalIsotopeAverageSp> &
mcql::Peptide::getPeptideNaturalIsotopeAverageSpList(
  pappso::PrecisionPtr precision, std::uint8_t charge, double ni_min_abundance)
{

  QMutexLocker lock(&m_mutex);
  if(m_niMinAbundance == ni_min_abundance)
    {
    }
  else
    {
      m_peptideNaturalIsotopeAverageSpListByCharge.clear();
      m_niMinAbundance = ni_min_abundance;
    }

  auto it = m_peptideNaturalIsotopeAverageSpListByCharge.find(charge);
  if(it != m_peptideNaturalIsotopeAverageSpListByCharge.end())
    {
      if(it->second.size() == 0)
        {
          it->second = mpa_peptideNaturalIsotopeList->getByIntensityRatio(
            charge, precision, ni_min_abundance);
        }

      return it->second;
    }
  else
    {
      auto it_insert = m_peptideNaturalIsotopeAverageSpListByCharge.insert(
        {charge,
         mpa_peptideNaturalIsotopeList->getByIntensityRatio(
           charge, precision, ni_min_abundance)});

      return it_insert.first->second;
    }
}

void
mcql::Peptide::addObservedChargeState(std::uint8_t charge)
{
  if(m_allObservedChargeStateList.end() ==
     std::find(m_allObservedChargeStateList.begin(),
               m_allObservedChargeStateList.end(),
               charge))
    {
      m_allObservedChargeStateList.push_back(charge);
    }
}

const std::vector<std::uint8_t> &
mcql::Peptide::getAllObservedChargeStateList() const
{
  return m_allObservedChargeStateList;
}

void
mcql::Peptide::addNotObservedInMsRunSp(const mcql::MsRunSp &msrun_sp)
{
  m_notObservedInMsRunSpList.push_back(msrun_sp);
}

const std::vector<mcql::ProteinSp> &
mcql::Peptide::getProteinSpList() const
{
  return m_proteinSpList;
}

void
mcql::Peptide::addAlignedPeakMeasurement(
  const PeptideMeasurements::Measurement &one_xic_measure,
  const pappso::MsRunRetentionTime<QString> &msrun_retention_time)
{
  if(one_xic_measure.m_tracePeakSp.get() != nullptr)
    {
      if(mp_referenceMsRunRetentionTime == &msrun_retention_time)
        {
          m_referenceMeasurementList.push_back(one_xic_measure);
        }
      AlignedPeakPositionElement position;
      position.intensity =
        one_xic_measure.m_tracePeakSp.get()->getMaxXicElement().y;
      if(msrun_retention_time.isAligned())
        {
          position.alignedRetentionTime =
            msrun_retention_time.translateOriginal2AlignedRetentionTime(
              one_xic_measure.m_tracePeakSp.get()->getMaxXicElement().x);
        }
      else
        {
          position.alignedRetentionTime =
            one_xic_measure.m_tracePeakSp.get()->getMaxXicElement().x;
        }

      position.wide =
        one_xic_measure.m_tracePeakSp.get()->getRightBoundary().x -
        one_xic_measure.m_tracePeakSp.get()->getLeftBoundary().x;
      m_alignedPeakPositionElementList.push_back(position);
    }
}

void
mcql::Peptide::computeConsensusRetentionTime()
{
  m_consensusAlignedRetentionTime = 0;

  if(m_alignedPeakPositionElementList.size() > 0)
    {
      std::sort(
        m_alignedPeakPositionElementList.begin(),
        m_alignedPeakPositionElementList.end(),
        [](AlignedPeakPositionElement const &a,
           AlignedPeakPositionElement const &b) { return a.wide > b.wide; });

      // take only the 50% best measures
      std::size_t limit = m_alignedPeakPositionElementList.size() / 2;
      if(limit < 5)
        {
          limit = m_alignedPeakPositionElementList.size();
          //  throw pappso::PappsoException("limit < 3");
        }
      double total_intensity =
        std::accumulate(m_alignedPeakPositionElementList.begin(),
                        m_alignedPeakPositionElementList.end() + limit,
                        (double)0.0,
                        [](double sum, AlignedPeakPositionElement &element) {
                          sum += element.intensity;
                          return (sum);
                        });
      double sum_rt =
        std::accumulate(m_alignedPeakPositionElementList.begin(),
                        m_alignedPeakPositionElementList.end() + limit,
                        (double)0.0,
                        [](double sum, AlignedPeakPositionElement &element) {
                          sum +=
                            element.intensity * element.alignedRetentionTime;
                          return (sum);
                        });

      m_consensusAlignedRetentionTime = sum_rt / total_intensity;
    }
}

bool
mcql::Peptide::isNotObservedInMsRunSp(const mcql::MsRun *msrun_p)
{
  auto it = std::find_if(
    m_notObservedInMsRunSpList.begin(),
    m_notObservedInMsRunSpList.end(),
    [msrun_p](const mcql::MsRunSp &a) { return (a.get() == msrun_p); });
  if(it == m_notObservedInMsRunSpList.end())
    return false;
  return true;
}

void
mcql::Peptide::setReferenceMsRunRetentionTimePtr(
  pappso::MsRunRetentionTime<QString> *msrun_retention_time_reference_p)
{
  mp_referenceMsRunRetentionTime = msrun_retention_time_reference_p;
}

double
mcql::Peptide::getConsensusRetentionTime() const
{
  return m_consensusAlignedRetentionTime;
}

void
mcql::Peptide::addMsRunXicCoordCharge(
  const mcql::Peptide::MsRunXicCoordCharge &msrun_xic_coord)
{
  m_msRunXicCoordChargeList.push_back(msrun_xic_coord);
}


pappso::XicCoordSPtr
mcql::Peptide::getBestXicCoordSPtrForChargeInMsRunGroup(
  const mcql::MsRunGroup *msrun_group_p, std::uint8_t charge) const
{
  // get XIC coordinates of the most intense precursor
  pappso::pappso_double intensity = -1;
  pappso::XicCoordSPtr best_xic_coord;

  for(auto &xic_coord_charge : m_msRunXicCoordChargeList)
    {
      if(msrun_group_p->contains(xic_coord_charge.msrun_p))
        {
          if(xic_coord_charge.charge == charge)
            {
              if(xic_coord_charge.intensity > intensity)
                {
                  intensity      = xic_coord_charge.intensity;
                  best_xic_coord = xic_coord_charge.xic_coord_sp;
                  qDebug();
                }
            }
        }
    }

  return best_xic_coord;
}


void
mcql::Peptide::populateIonMobilityGrid(
  pappso::IonMobilityGrid *ion_mobility_grid_p) const
{

  for(auto &xic_coord_charge_a : m_msRunXicCoordChargeList)
    {
      // qInfo() << " observed_in_a";
      const pappso::MsRunId &msrun_id_a =
        *(xic_coord_charge_a.msrun_p->getMsRunReaderSPtr()
            .get()
            ->getMsRunId()
            .get());
      pappso::XicCoordSPtr xic_coord_a = xic_coord_charge_a.xic_coord_sp;

      for(auto &xic_coord_charge_b : m_msRunXicCoordChargeList)
        {
          if(&xic_coord_charge_a == &xic_coord_charge_b)
            {
              break;
            }
          else
            {

              // qInfo() << " observed_in_b";
              const pappso::MsRunId &msrun_id_b =
                *(xic_coord_charge_b.msrun_p->getMsRunReaderSPtr()
                    .get()
                    ->getMsRunId()
                    .get());
              if(msrun_id_b == msrun_id_a)
                {
                }
              else
                {
                  pappso::XicCoordSPtr xic_coord_b =
                    xic_coord_charge_b.xic_coord_sp;

                  if(xic_coord_charge_a.charge == xic_coord_charge_b.charge)
                    {
                      ion_mobility_grid_p->storeObservedIdentityBetween(
                        msrun_id_a,
                        xic_coord_a.get(),
                        msrun_id_b,
                        xic_coord_b.get());
                    }
                }
            }
        }
    }
}


pappso::XicCoordSPtr
mcql::Peptide::getBestIonMobilityXicCoordToExtractOverallMsRunGroup(
  const mcql::MsRunGroup &msrun_group,
  const pappso::MsRunId &targeted_msrun,
  std::uint8_t charge) const
{
  pappso::XicCoordSPtr xic_coord_mean;
  std::size_t count = 0;
  pappso::IonMobilityGrid *ion_mobility_grid_p =
    msrun_group.getIonMobilityGridSp().get();

  for(auto &msrun_xic_coord_charge : m_msRunXicCoordChargeList)
    {
      if(msrun_group.contains(msrun_xic_coord_charge.msrun_p))
        {

          if(msrun_xic_coord_charge.charge == charge)
            {
              // compute the mean coordinate :
              pappso::XicCoordSPtr xic_coord_to_add =
                msrun_xic_coord_charge.xic_coord_sp;

              if(ion_mobility_grid_p != nullptr)
                {
                  //ion mobility coordinate translation for the targeted msrun
                  xic_coord_to_add =
                    ion_mobility_grid_p->translateXicCoordFromTo(
                      *xic_coord_to_add.get(),
                      *(msrun_xic_coord_charge.msrun_p->getMsRunReaderSPtr()
                          .get()
                          ->getMsRunId()
                          .get()),
                      targeted_msrun);
                }

              if(xic_coord_mean.get() == nullptr)
                {
                  xic_coord_mean = xic_coord_to_add.get()->initializeAndClone();
                }
              else
                {
                  xic_coord_mean =
                    xic_coord_mean.get()->addition(xic_coord_to_add);
                }
              count++;
              qDebug() << " xic_coord_mean.get()->toString="
                       << xic_coord_mean.get()->toString();
            }
        }
    }
  if(xic_coord_mean.get() != nullptr)
    {
      xic_coord_mean = xic_coord_mean.get()->divideBy(count);
    }


  return xic_coord_mean;
}
