/**
 * \file mcql/core/peptide.h
 * \date 24/09/2024
 * \author Olivier Langella
 * \brief peptide model in masschroqlite
 */

/*******************************************************************************
 * Copyright (c) 2024 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of MassChroQ.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#pragma once


#include <pappsomspp/peptide/peptidenaturalisotopelist.h>
#include "msrun.h"
#include "protein.h"
#include "peptidemeasurements.h"
#include <pappsomspp/msrun/alignment/msrunretentiontime.h>
#include <pappsomspp/msrun/xiccoord/ionmobilitygrid.h>
#include <cstdint>

namespace mcql
{

class Peptide;

typedef std::shared_ptr<Peptide> PeptideSp;

class MsRunGroup;

/**
 * unique studied entity representing the molecule we want to measure
 */
class Peptide
{
  public:
  /**
   * Default constructor
   */
  Peptide(const QString &id,
          pappso::PeptideSp msp_peptide,
          const std::vector<ProteinSp> &protein_list);

  /**
   * Copy constructor
   *
   * @param other TODO
   */
  Peptide(const Peptide &other);

  /**
   * Destructor
   */
  virtual ~Peptide();

  const pappso::PeptideSp &getPappsoPeptideSp() const;

  void computeIsotopologues(double ni_min_abundance);

  const pappso::PeptideNaturalIsotopeList *getPeptideNaturalIsotopeList() const;

  const QString &getId() const;

  void setMods(const QString &mods);
  const QString &getMods() const;

  const std::vector<pappso::PeptideNaturalIsotopeAverageSp> &
  getPeptideNaturalIsotopeAverageSpList(pappso::PrecisionPtr precision,
                                        std::uint8_t charge,
                                        double ni_min_abundance);

  void addObservedChargeState(std::uint8_t charge);
  void addNotObservedInMsRunSp(const mcql::MsRunSp &msrun_sp);

  const std::vector<std::uint8_t> &getAllObservedChargeStateList() const;

  const std::vector<ProteinSp> &getProteinSpList() const;

  void addAlignedPeakMeasurement(
    const PeptideMeasurements::Measurement &one_xic_measure,
    const pappso::MsRunRetentionTime<QString> &msrun_retention_time);

  void computeConsensusRetentionTime();


  struct AlignedPeakPositionElement
  {
    double alignedRetentionTime;
    double intensity;
    double wide;
  };


  bool isNotObservedInMsRunSp(const mcql::MsRun *msrun_p);

  pappso::XicCoordSPtr getBestXicCoordSPtrForChargeInMsRunGroup(
    const mcql::MsRunGroup *msrun_group_p, std::uint8_t charge) const;
  pappso::XicCoordSPtr getBestIonMobilityXicCoordToExtractOverallMsRunGroup(
    const mcql::MsRunGroup &msrun_group,
    const pappso::MsRunId &targeted_msrun,
    std::uint8_t charge) const;

  void setReferenceMsRunRetentionTimePtr(
    pappso::MsRunRetentionTime<QString> *msrun_retention_time_reference_p);

  double getConsensusRetentionTime() const;

  struct MsRunXicCoordCharge
  {
    mcql::MsRun *msrun_p;
    pappso::XicCoordSPtr xic_coord_sp;
    std::uint8_t charge;
    double intensity;
  };

  void addMsRunXicCoordCharge(const MsRunXicCoordCharge &msrun_xic_coord);


  void
  populateIonMobilityGrid(pappso::IonMobilityGrid *ion_mobility_grid_p) const;

  private:
  const QString m_id;
  const std::vector<ProteinSp> m_proteinSpList;
  QString m_mods;
  pappso::PeptideSp msp_peptide;
  pappso::PeptideNaturalIsotopeList *mpa_peptideNaturalIsotopeList = nullptr;
  std::map<std::uint8_t, std::vector<pappso::PeptideNaturalIsotopeAverageSp>>
    m_peptideNaturalIsotopeAverageSpListByCharge;
  double m_niMinAbundance = 0;
  std::vector<std::uint8_t> m_allObservedChargeStateList;
  std::vector<mcql::MsRunSp> m_notObservedInMsRunSpList;

  std::vector<AlignedPeakPositionElement> m_alignedPeakPositionElementList;

  double m_consensusAlignedRetentionTime = 0;

  pappso::MsRunRetentionTime<QString> *mp_referenceMsRunRetentionTime = nullptr;
  std::vector<mcql::PeptideMeasurements::Measurement>
    m_referenceMeasurementList;

  std::vector<MsRunXicCoordCharge> m_msRunXicCoordChargeList;
  QMutex m_mutex;
};
} // namespace mcql
